-- 
-- upgrade
-- 
-- @author Vinod Kurup (vinod@kurup.com)
-- @creation-date 2006-06-28
-- @arch-tag: 2443A272-06B7-11DB-8947-000393D14D28
-- @cvs-id $Id$
--

-- refresh trigger was calling old version of acs_object__new which ignored context_id and package_id

create or replace function dynamic_type__refresh_trigger (varchar)
returns integer as '
declare
  p_object_type           alias for $1;  
  insert_rule_text        text default '''';
  update_rule_text        text default '''';
  v_content_revision_p    boolean;       
  v_table_name            acs_object_types.table_name%TYPE;
  type_rec                record;
begin

  -- get the table name for the object type (determines view name)

  select table_name 
    into v_table_name
    from acs_object_types 
   where object_type = p_object_type;

  select content_type__is_content_type(p_object_type)
         into v_content_revision_p;
    
  --
  -- start building rule code
  --
  insert_rule_text := ''create rule '' || v_table_name || 
    ''_ir as on insert to '' || v_table_name || ''i do instead ('';

  update_rule_text := ''create rule '' || v_table_name || 
    ''_ur as on update to '' || v_table_name || ''i do instead ('';

  update_rule_text := update_rule_text || '' '' || dynamic_type__trigger_update_statement(''acs_object'') || '';'';

  -- if we are dealing with content revisions then add handler code

  if v_content_revision_p then
      insert_rule_text := insert_rule_text || ''update dt_dummy set val = (
                    select content_revision__new(
                                         new.title,
                                         new.description,
                                         now(),
                                         new.mime_type,
                                         new.nls_language,
                                         null,
                                         content_symlink__resolve(new.item_id),
                                         new.revision_id,
                                         now(),
                                         new.creation_user, 
                                         new.creation_ip
                    ));'';

      update_rule_text := update_rule_text || '' '' || dynamic_type__trigger_update_statement(''content_revision'') || '';'';

  else
      insert_rule_text := insert_rule_text || ''update dt_dummy set val = (
                    select acs_object__new(
                                         new.object_id,
                                         new.object_type,
                                         now(),
                                         new.creation_user, 
                                         new.creation_ip,
                                         new.context_id,
                                         ''''t'''',
                                         null,
                                         new.package_id
                    ));'';
  end if;

  -- add an insert statement for each subtype in the hierarchy for this type

  for type_rec in select ot2.object_type, tree_level(ot2.tree_sortkey) as level
                  from acs_object_types ot1, acs_object_types ot2
                  where ot1.object_type = p_object_type
                    and ot1.tree_sortkey between ot2.tree_sortkey and tree_right(ot2.tree_sortkey)
                  order by level asc
  loop
    if not (type_rec.object_type = ''acs_object'') then
      if not (type_rec.object_type = ''content_revision'') then
          insert_rule_text := insert_rule_text || '' '' || dynamic_type__trigger_insert_statement(type_rec.object_type) || '';'';
      end if;

      update_rule_text := update_rule_text || '' '' || dynamic_type__trigger_update_statement(type_rec.object_type) || '';'';
    end if;
  end loop;

  -- end building the rule definition code

  insert_rule_text := insert_rule_text || '' );'';
  update_rule_text := update_rule_text || '' );'';

  --
  -- done building rule code
  --

  -- drop the old rule
  if rule_exists(v_table_name || ''_ir'', v_table_name || ''i'') then 

    -- different syntax for dropping a rule in 7.2 and 7.3 so check which
    -- version is being used (olah).
    if version() like ''%7.2%'' then
      execute ''drop rule '' || v_table_name || ''_ir'';
      execute ''drop rule '' || v_table_name || ''_ur'';
    else
      -- 7.3 syntax
      execute ''drop rule '' || v_table_name || ''_ir on '' || v_table_name || ''i'';
      execute ''drop rule '' || v_table_name || ''_ur on '' || v_table_name || ''i'';
    end if;

  end if;

  -- create the new rule for inserts on the content type
  execute insert_rule_text;
  execute update_rule_text;

  return null; 

end;' language 'plpgsql';

-- we need a file widget
insert into dtype_widgets values ('file');
